/*   FILE: new.c -- A program to create a new file based on a template.
 * AUTHOR: W. Michael Petullo <new@flyn.org>
 *   DATE: 01 MAY 1 999
 *
 * Copyright (c) 1999 W. Michael Petullo <new@flyn.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <string.h>
#include <common.h>
#include <getopt.h>
#include <template.h>
#include <limits.h>
#include <pair.h>

extern char *usage_new;

typedef struct settings_t {
    int force;
    int list_only;
    int use_global;
    char *filepath;
    char template_path[PATH_MAX + 1];
    list_t cmd_line_kv;
} settings_t;

/* ============================ usage () ==================================== */
void usage(const int exitcode, const char *error, const char *more)
{
    fprintf (stderr, "%s\n", usage_new);
    if (error)
	fprintf(stderr, "%s: %s.\n\n", error, more);
    exit(exitcode);
}

/* ============================ parse_args () =============================== */
void parse_args(int argc, char *argv[], settings_t * settings)
{
    char c;
    int opt_index = 0;
    struct option opts[] = {
	{"help", 0, NULL, 'h'},
	{"force", 0, NULL, 'f'},
	{"list", 0, NULL, 'l'},
	{"global", 0, NULL, 'g'},
	{"template", 1, NULL, 't'},
	{"set", 1, NULL, 's'},
	{0, 0, 0, 0}
    };
    settings->force = 0;
    settings->list_only = 0;
    settings->use_global = 0;
    settings->filepath = NULL;
    strcpy (settings->template_path, "");
    list_init (&settings->cmd_line_kv, (void *) pair_destroy);
    while ((c = getopt_long(argc, argv, "hflgt:s:", opts, &opt_index)) >= 0) {
	switch (c) {
	pair_t *kv;
	case 'h':
	    usage(0, NULL, NULL);
	    exit(0);
	case 'f':
	    settings->force = 1;
	    break;
	case 'l':
	    settings->list_only = 1;
	    break;
	case 'g':
	    settings->use_global = 1;
	    break;
	case 't':
	    strncpy(settings->template_path, optarg,
		    sizeof(settings->template_path) - 1);
	    break;
	case 's':
	    kv = parse_kv (optarg);
	    if (kv)
	    	list_ins_next (&settings->cmd_line_kv, NULL, kv);
	    else {
                fprintf (stderr, "%s\n", usage_new);
		exit (1);
	    }
	    break;
	default:
            fprintf (stderr, "%s\n", usage_new);
	    exit(1);
	}
    }
    if (!(settings->filepath = argv[optind++]))
	usage(1, "new", "No filepath specified");
}

/* ============================ main () ===================================== */
int main(int argc, char *argv[], char *env[])
{
    settings_t settings;
    parse_args(argc, argv, &settings);
    template_init();
    if (!template_set_global_dir(GLOBAL_TEMPLATE_DIR)) {
	template_perror("Error setting global template dir");
	exit(1);
    }
    if (!template_set_local_dir(".new/templates")) {
	template_perror("Error setting global template dir");
	exit(1);
    }
    if (settings.list_only) {
	if (!template_list(settings.filepath)) {
	    template_perror("Error listing templates");
	    exit(1);
	}
    } else {
	if (! strlen(settings.template_path)) {
	    char template_path[PATH_MAX + 1];
	    char *template_name = argv && argv[optind] ? argv[optind] : "default";
	    const char *filename = basename(settings.filepath);
	    if (!template_find
		(template_path, filename, template_name,
		 settings.use_global)) {
		template_perror("error finding template");
		exit(1);
	    }
	    write_it_using_map(settings.filepath, settings.force, template_path, &settings.cmd_line_kv);
	} else
	    write_it_using_map(settings.filepath, settings.force,
		     settings.template_path, &settings.cmd_line_kv);
    }
    list_destroy (&settings.cmd_line_kv);
    template_destroy();
    exit(0);
}
